 package com.inet.ailink.receiver.service.impl;

 import com.inet.ailink.receiver.common.enums.StatusCode;
 import com.inet.ailink.receiver.common.utils.Base64TeaUitls;
 import com.inet.ailink.receiver.common.utils.DateUtils;
 import com.inet.ailink.receiver.common.utils.StringSelfUtil;
 import com.inet.ailink.receiver.common.utils.WeightUnitUtils;
 import com.inet.ailink.receiver.common.vo.Response;
 import com.inet.ailink.receiver.entity.*;
 import com.inet.ailink.receiver.mapper.*;
 import org.apache.commons.lang.StringUtils;
 import org.springframework.beans.factory.annotation.Autowired;
 import org.springframework.stereotype.Service;
 import org.springframework.transaction.annotation.Transactional;

 import javax.servlet.http.HttpServletRequest;
 import java.math.BigDecimal;
 import java.util.ArrayList;
 import java.util.Date;
 import java.util.List;
 import java.util.Map;
 import java.util.stream.Collectors;

 @Service
 @Transactional(rollbackFor=Exception.class)
 public class BodyFatServiceImpl extends  BaseServiceImpl<IBodyFatDao,BodyFat>
 {

     @Autowired
     IBodyFatDao bodyFatDao;


     @Override
     protected IBodyFatDao getEntityDao() {
         return bodyFatDao;
     }

     public Response<Object> saveWeighDataByAdc(byte[] paramsByte, HttpServletRequest request){
         Response<Object> result = new Response<Object>();
         //解析数据
         //设备ID
         Integer deviceId = Integer.parseInt(Base64TeaUitls.getLowConactHighEndForInt(0, 3, paramsByte));
         //设备时间
         String devivceTimeStr = Base64TeaUitls.getStartToEndForInt(4,9, paramsByte);
         String[] devivceTimeArry = devivceTimeStr.split(":");
         String deviceTimeFormate = (2000+Integer.parseInt(devivceTimeArry[0]))+"-"+devivceTimeArry[1]+"-"+devivceTimeArry[2]+" "+devivceTimeArry[3]+":"+devivceTimeArry[4]+":"+devivceTimeArry[5];
         Date devivceTime = DateUtils.parse(deviceTimeFormate, DateUtils.DATETIME_FORMAT_PATTERN);
         //如果设备端时间为空，时间会传0，判断月份是否为0（正常情况，月份不存在为0的情况），为0，则使用当前服务器时间
         if(!StringSelfUtil.isEmpty(devivceTimeArry[1]) && Integer.parseInt(devivceTimeArry[1]) == 0){
             //获取零时区时间
             devivceTime =DateUtils.getCurrZoneGMT0();
         }
         //设备在线状态
         String deviceOnlineStatus = Base64TeaUitls.getStartToEndForInt(10, 10, paramsByte);
         //体重
         String deviceWeigh = Base64TeaUitls.getLowConactHighEndForInt(11, 13, paramsByte);
         //体重单位
         String deviceWeighUnit = Base64TeaUitls.getStartToEndForInt(14, 14, paramsByte);
         //体重精度
         String deviceWeighPoint = Base64TeaUitls.getStartToEndForInt(15, 15, paramsByte);
         //阻抗
         String deviceAdc = Base64TeaUitls.getLowConactHighEndForInt(16, 17, paramsByte);
         //心率
         String deviceHeartRate = Base64TeaUitls.getStartToEndForInt(18, 18, paramsByte);
         //算法
         String deviceAlgorithm  = Base64TeaUitls.getStartToEndForInt(19, 19, paramsByte);
         //用户id
         Integer subUserId = Integer.parseInt(Base64TeaUitls.getLowConactHighEndForInt(20, 23, paramsByte));
         //将解析出的数据，赋值
         BodyFat bodyFat = new BodyFat(deviceId, deviceWeigh, deviceWeighUnit, deviceWeighPoint, deviceAdc, devivceTime, deviceHeartRate, deviceAlgorithm,subUserId);
         bodyFat.setUploadTime(DateUtils.currentDate());
         //如果设备传的时间不为空，则创建时间为设备的时间，反之，使用当前时间
         Date beginDate = DateUtils.parse("1970-01-01 00:00:00");
         bodyFat.setCreateTimeLong(DateUtils.getTimeDiff(beginDate, devivceTime)*1000);

         return saveAndPushBodyFat(bodyFat);
     }


     /**
      * 2021-01-19更新体脂数据保存以及推送策略，同一个WIFI体脂秤可以多个账户绑定，
      * @param bodyFat
      * @return
      */
     public Response<Object> saveAndPushBodyFat(BodyFat bodyFat){
         Response<Object> result = new Response<Object>();
         bodyFat.setAppUserId(0);
         bodyFat.setSubUserId(0);
         result = saveBodyFat(bodyFat);
         return result;

     }

     //保存体重数据
     public Response<Object> saveBodyFat(BodyFat bodyFat){
         Response<Object> result = new Response<Object>();
         int updateResult = 0;
         //查询用户并且计算出其他身体指标
         //bodyFat = getUserBodyFatByAppuserId(bodyFat);
         //增加数据来源与计算次数，APP收到消息后，进行数据计算
         bodyFat.setSource(1);
         bodyFat.setCalculationTimes(0);
         System.out.println("bodyFat = " + bodyFat);

         //将当前wifi秤上传的重量(可能为kg/st/lb/斤)，转换为kg数值
         BigDecimal kgWeight = WeightUnitUtils.getKg(new BigDecimal(bodyFat.getWeight()) ,Integer.parseInt(bodyFat.getWeightUnit()),Integer.parseInt(bodyFat.getWeightPoint()));
         System.out.println("kgWeight = " + kgWeight);

         if(bodyFat.getBodyFatId() == null){//新增体重数据
             updateResult = bodyFatDao.insert(bodyFat);
         }else{//修改体重数据
             updateResult = bodyFatDao.update(bodyFat);
         }
         if(updateResult>0){
             result.setStatus(StatusCode.SUCCESS.getCode());
             result.setData(bodyFat.getBodyFatId());
         }else{
             result.setStatus(StatusCode.FAIL.getCode());
         }
         return result;
     }




 }